(* ************************************************************************* *)
(*                             TinyQCMl                                      *)
(*                   (c) Franois Pessaux - 2014                             *)
(* TinyQCMl is freely distribuable (sources and binaries if some).           *)
(* ************************************************************************* *)

exception Lex_error of Lexing.position * Lexing.position * string ;;
exception Syntax_error of Lexing.position * Lexing.position ;;
exception Unclear_error of string * Lexing.position * Lexing.position ;;

let pp_err_header ppf = Printf.fprintf ppf "Error: " ;;


let pp_err_loc ppf (sp, ep) =
  Printf.fprintf ppf
    "File %S, line %i, characters %i-%i"
    sp.Lexing.pos_fname
    sp.Lexing.pos_lnum
    (sp.Lexing.pos_cnum - sp.Lexing.pos_bol)
    (ep.Lexing.pos_cnum - sp.Lexing.pos_bol)
;;



let wrap_parse lex_buf =
  let ast = Parser.whole_file Lexer.token lex_buf in
  Parsing.clear_parser () ;
  ast
;;



let parse in_file_name =
  let in_hd = open_in_bin in_file_name in
  let lex_buff = Lexing.from_channel in_hd in
  lex_buff.Lexing.lex_curr_p <-
    { Lexing.pos_fname = in_file_name ; pos_lnum = 1 ;
      pos_bol = 0 ; pos_cnum = 0 } ;
  try
    let questions =
      try wrap_parse lex_buff with
      | x ->
          close_in in_hd ;
          raise x in
    close_in in_hd ;
    questions
  with
  | Lexer.Error (err, sp, ep) ->
      raise (Lex_error (sp, ep, (Lexer.string_of_lex_error err)))
  | Parsing.Parse_error ->
      let sp = Lexing.lexeme_start_p lex_buff in
      let ep = Lexing.lexeme_end_p lex_buff in
      raise (Syntax_error (sp, ep));
  | x ->
      let sp = Lexing.lexeme_start_p lex_buff in
      let ep = Lexing.lexeme_end_p lex_buff in
      raise (Unclear_error (Printexc.to_string x, sp, ep))
;;
