/* ************************************************************************* */
/*                             TinyQCMl                                      */
/*                   (c) Franois Pessaux - 2014                             */
/* TinyQCMl is freely distribuable (sources and binaries if some).           */
/* ************************************************************************* */

%{
%}

%token EOF

/* Nested symbols */
%token LBRACE
%token RBRACE

/* General infix and prefix operators */
%token COMMA
%token EQUAL

/* Keywords */
%token KO
%token OK

/* Separators. */
%token SEMI

/* Strings. */
%token <string> STRING

  /* Floats. */
%token <float> FLOAT


%start whole_file
%type <Parsetree.file_t> whole_file
%%

whole_file:
  | phrases { Array.of_list $1 }
;

phrases:
  | EOF               { [] }
  | phrase phrases    { $1 :: $2 }
;

phrase:
  | LBRACE items_or_free_text RBRACE    { $2 }
;

items_or_free_text:
  | items       { Parsetree.QChoices (Array.of_list $1) }
  | free_texts  { Parsetree.QFree (Array.of_list $1) }
;

items:
  | STRING SEMI non_empty_answers opt_alt_items
      { { Parsetree.i_text = $1 ; Parsetree.i_props = $3 } :: $4 }
;

non_empty_answers:
  | LBRACE answer answers RBRACE
      { Array.of_list ($2 :: $3) }
;

answers:
  | /* Empty. */     { [] }
  | answer answers    { $1 :: $2 }
;

answer:
  | STRING COMMA validity SEMI
      { { Parsetree.p_text = $1 ; Parsetree. p_valid = $3 }}
;

validity:
  | OK { true }
  | KO { false }
;

opt_alt_items:
  | /* Empty. */    { [] }
  | items           { $1 }
;


free_texts:
  | STRING COMMA FLOAT COMMA STRING SEMI opt_alt_free_texts
      {
       { Parsetree.ft_text = $1 ;
         Parsetree.ft_frame_height = $3 ;
         Parsetree.ft_answer = $5 } :: $7
     }
;

opt_alt_free_texts:
  | /* Empty. */    { [] }
  | free_texts      { $1 }
;
