(* ************************************************************************* *)
(*                             TinyQCMl                                      *)
(*                   (c) Franois Pessaux - 2014                             *)
(* TinyQCMl is freely distribuable (sources and binaries if some).           *)
(* ************************************************************************* *)

(** {b Descr}: Proposition for a multiple choices question. *)
type proposition_t = {
  p_text : string ;  (** Text of the proposition. *)
  p_valid : bool ;   (** Validity of the proposition. *)
} ;;

(** {b Descr}: Multiple choices question. *)
type item_t = {
  i_text : string ;                (** Text of the question. *)
  i_props : proposition_t array ;  (** Propositions of the question. *)
} ;;

(** {b Descr}: Free text question. *)
type free_text_t = {
  ft_text : string ;         (** Text of the question. *)
  ft_frame_height : float ;  (** Size of the vertical box left for answering. *)
  ft_answer : string         (** Answer of the question. *)
} ;;

(** {b Descr}: A question being compoound of several variants. *)
type question_t =
  | QChoices of item_t array    (** Multiple choice kind. *)
  | QFree of free_text_t array  (** Free text kind. *)
;;

(** {b Descr}: A whole exam. In fact, a bunch of questions. *)
type file_t = question_t array ;;
